@@ -44,7 +44,6 @@
 #include "body.h"
 #include "cell.h"
 #include "image.h"
-#include "link.h"
 #include "menubar.h"
 #include "popup.h"
 #include "properties.h"
@@ -82,7 +81,7 @@ remove_link (GtkWidget *mi, GtkHTMLControlData *cd)
 	html_engine_selection_push (cd->html->engine);
 	if (!html_engine_is_selection_active (cd->html->engine))
 		html_engine_select_word_editable (cd->html->engine);
-	html_engine_insert_link (cd->html->engine, NULL, NULL);
+	html_engine_edit_set_link (cd->html->engine, NULL, NULL);
 	html_engine_selection_pop (cd->html->engine);
 }
 
@@ -147,7 +146,7 @@ show_prop_dialog (GtkHTMLControlData *cd, GtkHTMLEditPropertyType start)
 	GList *cur;
 
 	if (cd->properties_dialog)
-		gtk_html_edit_properties_dialog_destroy (cd->properties_dialog);
+		gtk_html_edit_properties_dialog_close (cd->properties_dialog);
 	cd->properties_dialog = gtk_html_edit_properties_dialog_new (cd, FALSE, _("Properties"));
 
 	cur = cd->properties_types;
@@ -161,6 +160,13 @@ show_prop_dialog (GtkHTMLControlData *cd, GtkHTMLEditPropertyType start)
 								   text_apply_cb,
 								   text_close_cb);
 			break;
+		case GTK_HTML_EDIT_PROPERTY_LINK:
+			gtk_html_edit_properties_dialog_add_entry (cd->properties_dialog,
+								   t, _("Link"),
+								   link_properties,
+								   link_apply_cb,
+								   link_close_cb);
+			break;
 		case GTK_HTML_EDIT_PROPERTY_IMAGE:
 			gtk_html_edit_properties_dialog_add_entry (cd->properties_dialog,
 								   t, _("Image"),
@@ -175,13 +181,6 @@ show_prop_dialog (GtkHTMLControlData *cd, GtkHTMLEditPropertyType start)
 								   paragraph_apply_cb,
 								   paragraph_close_cb);
 			break;
-		case GTK_HTML_EDIT_PROPERTY_LINK:
-			gtk_html_edit_properties_dialog_add_entry (cd->properties_dialog,
-								   t, _("Link"),
-								   link_properties,
-								   link_apply_cb,
-								   link_close_cb);
-			break;
 		case GTK_HTML_EDIT_PROPERTY_BODY:
 			gtk_html_edit_properties_dialog_add_entry (cd->properties_dialog,
 								   t, _("Page"),
@@ -227,6 +226,24 @@ prop_dialog (GtkWidget *mi, GtkHTMLControlData *cd)
 	show_prop_dialog (cd, GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (mi), "type")));
 }
 
+static void
+link_prop_dialog (GtkWidget *mi, GtkHTMLControlData *cd)
+{
+	if (cd->properties_dialog)
+		gtk_html_edit_properties_dialog_close (cd->properties_dialog);
+
+	cd->properties_dialog = gtk_html_edit_properties_dialog_new (cd, FALSE, _("Properties"));
+
+	gtk_html_edit_properties_dialog_add_entry (cd->properties_dialog,
+						   GTK_HTML_EDIT_PROPERTY_LINK, _("Link"),
+						   link_properties,
+						   link_apply_cb,
+						   link_close_cb);
+
+	gtk_html_edit_properties_dialog_show (cd->properties_dialog);
+	gtk_html_edit_properties_dialog_set_page (cd->properties_dialog, GTK_HTML_EDIT_PROPERTY_LINK);
+}
+
 static void
 spell_suggest (GtkWidget *mi, GtkHTMLControlData *cd)
 {
@@ -407,16 +424,23 @@ prepare_properties_and_menu (GtkHTMLControlData *cd, guint *items)
 		}
 		ADD_ITEM (_("Paste"),  paste, NONE);
 	}
-	if (cd->format_html && html_engine_is_selection_active (e)) {
-		ADD_SEP;
-		ADD_ITEM (_("Text..."), prop_dialog, TEXT);
-		ADD_PROP (TEXT);
-		ADD_ITEM (_("Paragraph..."), prop_dialog, PARAGRAPH);
-		ADD_PROP (PARAGRAPH);
-		ADD_ITEM (_("Link..."), prop_dialog, LINK);
-		ADD_PROP (LINK);
-	} else if (cd->format_html && obj) {
+
+	if (cd->format_html && obj) {
 		switch (HTML_OBJECT_TYPE (obj)) {
+		case HTML_TYPE_TEXT:
+			ADD_SEP;
+			ADD_ITEM (_("Text..."), prop_dialog, TEXT);
+			ADD_PROP (TEXT);
+			ADD_ITEM (_("Paragraph..."), prop_dialog, PARAGRAPH);
+			ADD_PROP (PARAGRAPH);
+			break;
+		case HTML_TYPE_LINKTEXT:
+			ADD_SEP;
+			ADD_ITEM (_("Link..."), link_prop_dialog, LINK);
+			ADD_PROP (LINK);
+			ADD_ITEM (_("Paragraph..."), prop_dialog, PARAGRAPH);
+			ADD_PROP (PARAGRAPH);
+			break;
 		case HTML_TYPE_RULE:
 			ADD_SEP;
 			ADD_ITEM (_("Rule..."), prop_dialog, RULE);
@@ -428,18 +452,6 @@ prepare_properties_and_menu (GtkHTMLControlData *cd, guint *items)
 			ADD_PROP (IMAGE);
 			ADD_ITEM (_("Paragraph..."), prop_dialog, PARAGRAPH);
 			ADD_PROP (PARAGRAPH);
-			ADD_ITEM (_("Link..."), prop_dialog, LINK);
-			ADD_PROP (LINK);
-			break;
-		case HTML_TYPE_LINKTEXT:
-		case HTML_TYPE_TEXT:
-			ADD_SEP;
-			ADD_PROP (TEXT);
-			ADD_ITEM (_("Text..."), prop_dialog, TEXT);
-			ADD_PROP (PARAGRAPH);
-			ADD_ITEM (_("Paragraph..."), prop_dialog, PARAGRAPH);
-			ADD_PROP (LINK);
-			ADD_ITEM (_("Link..."), prop_dialog, LINK);
 			break;
 		case HTML_TYPE_TABLE:
 			ADD_SEP;
@@ -501,6 +513,34 @@ popup_show (GtkHTMLControlData *cd, GdkEventButton *event)
 	return (items > 0);
 }
 
+static void
+set_position (GtkMenu *menu, gint *x, gint *y, gpointer data)
+{
+	GtkHTMLControlData *cd = (GtkHTMLControlData *) data;
+	HTMLEngine *e = cd->html->engine;
+	gint xw, yw;
+
+	gdk_window_get_origin (GTK_WIDGET (cd->html)->window, &xw, &yw);
+	html_object_get_cursor_base (e->cursor->object, e->painter, e->cursor->offset, x, y);
+	*x += xw + e->leftBorder;
+	*y += yw + e->topBorder;
+}
+
+gint
+popup_show_at_cursor (GtkHTMLControlData *cd)
+{
+	GtkWidget *menu;
+	guint items = 0;
+
+	menu = prepare_properties_and_menu (cd, &items);
+	gtk_widget_show (menu);
+	if (items)
+		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, set_position, cd, 0, 0);
+	gtk_widget_unref (menu);
+
+	return (items > 0);
+}
+
 void
 property_dialog_show (GtkHTMLControlData *cd)
 {
