@@ -22,6 +22,7 @@
 
 
 #include <config.h>
+#include "dialog.h"
 #include "properties.h"
 
 #define GTK_HTML_EDIT_IMAGE_BWIDTH      0
@@ -109,6 +110,7 @@ GtkHTMLEditPropertiesDialog *
 gtk_html_edit_properties_dialog_new (GtkHTMLControlData *cd, gboolean insert, gchar *title)
 {
 	GtkHTMLEditPropertiesDialog *d = g_new (GtkHTMLEditPropertiesDialog, 1);
+	GtkWindow *parent;
 
 	d->page_data      = NULL;
 	d->title          = g_strdup (title);
@@ -121,6 +123,10 @@ gtk_html_edit_properties_dialog_new (GtkHTMLControlData *cd, gboolean insert, gc
 				     GNOME_STOCK_BUTTON_OK,
 				     GNOME_STOCK_BUTTON_APPLY,
 				     GNOME_STOCK_BUTTON_CLOSE, NULL);
+	parent = get_parent_window (GTK_WIDGET (cd->html));
+	if (parent) {
+		gnome_dialog_set_parent (GNOME_DIALOG (d->dialog), parent);
+	}
 	d->notebook = gtk_notebook_new ();
 	gtk_signal_connect (GTK_OBJECT (d->dialog), "destroy", destroy_dialog, d);
 	gtk_signal_connect (GTK_OBJECT (d->notebook), "switch_page", switch_page, d);
