@@ -49,6 +49,10 @@ static struct {
 } paragraph_style_items[] = {
 	{ GTK_HTML_PARAGRAPH_STYLE_NORMAL, N_("Normal") },
 	{ GTK_HTML_PARAGRAPH_STYLE_PRE, N_("Preformat") },
+	{ GTK_HTML_PARAGRAPH_STYLE_ITEMDOTTED, N_("Bulleted List") },
+	{ GTK_HTML_PARAGRAPH_STYLE_ITEMDIGIT, N_("Numbered List") },
+	{ GTK_HTML_PARAGRAPH_STYLE_ITEMROMAN, N_("Roman List") },
+	{ GTK_HTML_PARAGRAPH_STYLE_ITEMALPHA, N_("Alphabetical List") },
 	{ GTK_HTML_PARAGRAPH_STYLE_H1, N_("Header 1") },
 	{ GTK_HTML_PARAGRAPH_STYLE_H2, N_("Header 2") },
 	{ GTK_HTML_PARAGRAPH_STYLE_H3, N_("Header 3") },
@@ -56,9 +60,6 @@ static struct {
 	{ GTK_HTML_PARAGRAPH_STYLE_H5, N_("Header 5") },
 	{ GTK_HTML_PARAGRAPH_STYLE_H6, N_("Header 6") },
 	{ GTK_HTML_PARAGRAPH_STYLE_ADDRESS, N_("Address") },
-	{ GTK_HTML_PARAGRAPH_STYLE_ITEMDIGIT, N_("List item (digit)") },
-	{ GTK_HTML_PARAGRAPH_STYLE_ITEMDOTTED, N_("List item (unnumbered)") },
-	{ GTK_HTML_PARAGRAPH_STYLE_ITEMROMAN, N_("List item (roman)") },
 	{ GTK_HTML_PARAGRAPH_STYLE_NORMAL, NULL },
 };
 
@@ -111,6 +112,7 @@ paragraph_style_menu_item_update (GtkWidget *widget, gpointer format_html)
 		     || style == GTK_HTML_PARAGRAPH_STYLE_ITEMDOTTED
 		     || style == GTK_HTML_PARAGRAPH_STYLE_ITEMROMAN
 		     || style == GTK_HTML_PARAGRAPH_STYLE_ITEMDIGIT
+		     || style == GTK_HTML_PARAGRAPH_STYLE_ITEMALPHA
 		     );
 
 	gtk_widget_set_sensitive (widget, sensitive);	
@@ -177,10 +179,10 @@ font_size_changed (GtkWidget *w, GtkHTMLParagraphStyle style, GtkHTMLControlData
 {
 	if (style == GTK_HTML_FONT_STYLE_DEFAULT)
 		style = GTK_HTML_FONT_STYLE_SIZE_3;
-	cd->block_font_style_change = TRUE;
+	cd->block_font_style_change++;
 	gtk_option_menu_set_history (GTK_OPTION_MENU (cd->font_size_menu),
 				     (style & GTK_HTML_FONT_STYLE_SIZE_MASK) - GTK_HTML_FONT_STYLE_SIZE_1);
-	cd->block_font_style_change = FALSE;
+	cd->block_font_style_change--;
 }
 
 static GtkWidget *
@@ -225,9 +227,11 @@ setup_font_size_option_menu (GtkHTMLControlData *cd)
 }
 
 static void
-color_changed (GtkWidget *w, GdkColor *gdk_color, GtkHTMLControlData *cd)
+apply_color (GdkColor *gdk_color, GtkHTMLControlData *cd)
 {
-	HTMLColor *color = gdk_color
+	HTMLColor *color;
+	
+	color = gdk_color
 		&& gdk_color != &html_colorset_get_color (cd->html->engine->settings->color_set, HTMLTextColor)->color
 		? html_color_new_from_gdk_color (gdk_color) : NULL;
 
@@ -236,13 +240,35 @@ color_changed (GtkWidget *w, GdkColor *gdk_color, GtkHTMLControlData *cd)
 		html_color_unref (color);
 }
 
+void
+toolbar_apply_color (GtkHTMLControlData *cd)
+{
+	GdkColor *color;
+
+	color = color_combo_get_color (COLOR_COMBO (cd->combo));
+	apply_color (color, cd);
+	if (color)
+		gdk_color_free (color);
+}
+
+static void
+color_changed (GtkWidget *w, GdkColor *gdk_color, gboolean by_user, GtkHTMLControlData *cd)
+{
+	/* If the color was changed programatically there's not need to set things */
+	if (!by_user)
+		return;
+	apply_color (gdk_color, cd);
+}
+
 static void
 unset_focus (GtkWidget *w, gpointer data)
 {
 	GTK_WIDGET_UNSET_FLAGS (w, GTK_CAN_FOCUS);
+	if (GTK_IS_CONTAINER (w))
+		gtk_container_forall (GTK_CONTAINER (w), unset_focus, NULL);
 }
 
-inline static void
+static void
 set_color_combo (GtkHTML *html, GtkHTMLControlData *cd)
 {
 	color_combo_set_color (COLOR_COMBO (cd->combo),
@@ -277,7 +303,7 @@ setup_color_combo (GtkHTMLControlData *cd)
 		gtk_signal_connect (GTK_OBJECT (cd->html), "realize", realize_engine, cd);
         gtk_signal_connect (GTK_OBJECT (cd->html), "load_done", GTK_SIGNAL_FUNC (load_done), cd);
 
-	cd->combo = color_combo_new (NULL, _("Automatic"), &color->color, "toolbar_text");
+	cd->combo = color_combo_new (NULL, _("Automatic"), &color->color, color_group_fetch ("toolbar_text", cd));
 	GTK_WIDGET_UNSET_FLAGS (cd->combo, GTK_CAN_FOCUS);
 	gtk_container_forall (GTK_CONTAINER (cd->combo), unset_focus, NULL);
         gtk_signal_connect (GTK_OBJECT (cd->combo), "changed", GTK_SIGNAL_FUNC (color_changed), cd);
@@ -357,7 +383,7 @@ editor_toolbar_strikeout_cb (GtkWidget *widget, GtkHTMLControlData *cd)
 static void
 insertion_font_style_changed_cb (GtkHTML *widget, GtkHTMLFontStyle font_style, GtkHTMLControlData *cd)
 {
-	cd->block_font_style_change = TRUE;
+	cd->block_font_style_change++;
 
 	if (font_style & GTK_HTML_FONT_STYLE_FIXED)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cd->tt_button), TRUE);
@@ -384,7 +410,7 @@ insertion_font_style_changed_cb (GtkHTML *widget, GtkHTMLFontStyle font_style, G
 	else
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cd->strikeout_button), FALSE);
 
-	cd->block_font_style_change = FALSE;
+	cd->block_font_style_change--;
 }
 
 
@@ -468,17 +494,18 @@ paragraph_alignment_changed_cb (GtkHTML *widget,
 /* Indentation group.  */
 
 static void
+
 editor_toolbar_indent_cb (GtkWidget *widget,
 			  GtkHTMLControlData *cd)
 {
-	gtk_html_modify_indent_by_delta (GTK_HTML (cd->html), +1);
+	gtk_html_indent_push_level (GTK_HTML (cd->html), HTML_LIST_TYPE_BLOCKQUOTE);
 }
 
 static void
 editor_toolbar_unindent_cb (GtkWidget *widget,
 			    GtkHTMLControlData *cd)
 {
-	gtk_html_modify_indent_by_delta (GTK_HTML (cd->html), -1);
+	gtk_html_indent_pop_level (GTK_HTML (cd->html));
 }
 
 
@@ -487,7 +514,7 @@ editor_toolbar_unindent_cb (GtkWidget *widget,
 static GnomeUIInfo editor_toolbar_alignment_group[] = {
 	GNOMEUIINFO_ITEM_STOCK (N_("Left align"), N_("Left justifies the paragraphs"),
 				editor_toolbar_left_align_cb, GNOME_STOCK_PIXMAP_ALIGN_LEFT),
-	GNOMEUIINFO_ITEM_STOCK (N_("Center"), N_("Centers justifies the paragraphs"),
+	GNOMEUIINFO_ITEM_STOCK (N_("Center"), N_("Center justifies the paragraphs"),
 				editor_toolbar_center_cb, GNOME_STOCK_PIXMAP_ALIGN_CENTER),
 	GNOMEUIINFO_ITEM_STOCK (N_("Right align"), N_("Right justifies the paragraphs"),
 				editor_toolbar_right_align_cb, GNOME_STOCK_PIXMAP_ALIGN_RIGHT),
@@ -549,7 +576,7 @@ create_style_toolbar (GtkHTMLControlData *cd)
 	cd->toolbar_style = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
 
 	gtk_container_add (GTK_CONTAINER (frame), cd->toolbar_style);
-	gtk_box_pack_start (GTK_BOX (hbox), frame, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), frame, TRUE, TRUE, 0);
 
 	gtk_widget_show_all (hbox);
 
@@ -606,7 +633,10 @@ toolbar_item_update_sensitivity (GtkWidget *widget, gpointer data)
 	sensitive = (cd->format_html
 		     || widget == cd->paragraph_option
 		     || widget == cd->indent_button
-		     || widget == cd->unindent_button);
+		     || widget == cd->unindent_button
+		     || widget == cd->left_align_button
+		     || widget == cd->center_button
+		     || widget == cd->right_align_button);
 
 	gtk_widget_set_sensitive (widget, sensitive);
 }
