@@ -28,6 +28,7 @@ typedef enum {
 	GTK_HTML_COMMAND_UNDO,
 	GTK_HTML_COMMAND_REDO,
 	GTK_HTML_COMMAND_COPY,
+	GTK_HTML_COMMAND_COPY_AND_DISABLE_SELECTION,
 	GTK_HTML_COMMAND_CUT,
 	GTK_HTML_COMMAND_PASTE,
 
@@ -39,7 +40,7 @@ typedef enum {
 	GTK_HTML_COMMAND_INSERT_RULE_PARAM,
 	GTK_HTML_COMMAND_INSERT_IMAGE_PARAM,
 	GTK_HTML_COMMAND_INSERT_TAB_OR_INDENT_MORE,
-	GTK_HTML_COMMAND_INSERT_TAB_OR_INDENT_MORE_OR_NEXT_CELL,
+	GTK_HTML_COMMAND_INSERT_TAB_OR_NEXT_CELL,
 
 	GTK_HTML_COMMAND_MAKE_LINK,
 	GTK_HTML_COMMAND_REMOVE_LINK,
@@ -84,8 +85,9 @@ typedef enum {
 
 	GTK_HTML_COMMAND_INDENT_ZERO,
 	GTK_HTML_COMMAND_INDENT_INC,
+	GTK_HTML_COMMAND_INDENT_INC_OR_NEXT_CELL,
 	GTK_HTML_COMMAND_INDENT_DEC,
-	GTK_HTML_COMMAND_INDENT_DEC_OR_PREV_CELL,
+	GTK_HTML_COMMAND_PREV_CELL,
 	GTK_HTML_COMMAND_INDENT_PARAGRAPH,
 
 	GTK_HTML_COMMAND_BREAK_AND_FILL_LINE,
@@ -103,6 +105,7 @@ typedef enum {
 	GTK_HTML_COMMAND_PARAGRAPH_STYLE_ITEMDOTTED,
 	GTK_HTML_COMMAND_PARAGRAPH_STYLE_ITEMROMAN,
 	GTK_HTML_COMMAND_PARAGRAPH_STYLE_ITEMDIGIT,
+	GTK_HTML_COMMAND_PARAGRAPH_STYLE_ITEMALPHA,
 
 	GTK_HTML_COMMAND_MODIFY_SELECTION_UP,
 	GTK_HTML_COMMAND_MODIFY_SELECTION_DOWN,
@@ -195,6 +198,22 @@ typedef enum {
 	GTK_HTML_COMMAND_DELETE_TABLE_ROW,
 	GTK_HTML_COMMAND_DELETE_TABLE_COLUMN,
 	GTK_HTML_COMMAND_DELETE_TABLE_CELL_CONTENTS,
+
+	GTK_HTML_COMMAND_GRAB_FOCUS,
+
+	GTK_HTML_COMMAND_KILL_WORD,
+	GTK_HTML_COMMAND_KILL_WORD_BACKWARD,
+
+	GTK_HTML_COMMAND_TEXT_COLOR_APPLY,
+
+	GTK_HTML_COMMAND_SAVE_DATA_ON,
+	GTK_HTML_COMMAND_SAVE_DATA_OFF,
+
+	GTK_HTML_COMMAND_SAVED,
+	GTK_HTML_COMMAND_IS_SAVED,
+
+	GTK_HTML_COMMAND_SCROLL_BOD,
+	GTK_HTML_COMMAND_SCROLL_EOD,
 } GtkHTMLCommandType;
 
 typedef enum {
@@ -205,8 +224,10 @@ typedef enum {
 } GtkHTMLCursorSkipType;
 
 typedef enum {
-	GTK_HTML_EDITOR_EVENT_COMMAND,
-	GTK_HTML_EDITOR_EVENT_IMAGE_URL
+	GTK_HTML_EDITOR_EVENT_COMMAND_BEFORE,
+	GTK_HTML_EDITOR_EVENT_COMMAND_AFTER,
+	GTK_HTML_EDITOR_EVENT_IMAGE_URL,
+	GTK_HTML_EDITOR_EVENT_DELETE
 } GtkHTMLEditorEventType;
 
 typedef enum {
@@ -252,7 +273,8 @@ typedef enum {
 	GTK_HTML_PARAGRAPH_STYLE_PRE,
 	GTK_HTML_PARAGRAPH_STYLE_ITEMDOTTED,
 	GTK_HTML_PARAGRAPH_STYLE_ITEMROMAN,
-	GTK_HTML_PARAGRAPH_STYLE_ITEMDIGIT
+	GTK_HTML_PARAGRAPH_STYLE_ITEMDIGIT,
+	GTK_HTML_PARAGRAPH_STYLE_ITEMALPHA
 } GtkHTMLParagraphStyle;
 
 typedef enum {
