@@ -40,10 +40,8 @@
 
 /* This routine was originally written by Daniel Velliard, (C) 1998 World Wide
    Web Consortium.  */
-static gchar *
-encode_entities (const gchar *input,
-		 guint len,
-		 guint *encoded_len_return)
+gchar *
+html_encode_entities (const gchar *input, guint len, guint *encoded_len_return)
 {
 	gunichar uc;
 	const gchar *p;
@@ -121,7 +119,8 @@ encode_entities (const gchar *input,
 	}
 
 	*out = 0;
-	*encoded_len_return = out - buffer;
+	if (encoded_len_return)
+		*encoded_len_return = out - buffer;
 
 	return buffer;
 }
@@ -133,6 +132,7 @@ html_engine_save_encode (HTMLEngineSaveState *state,
 {
 	guchar *encoded_buffer;
 	guint encoded_length;
+	gboolean success;
 
 	g_return_val_if_fail (state != NULL, FALSE);
 	g_return_val_if_fail (buffer != NULL, FALSE);
@@ -140,9 +140,11 @@ html_engine_save_encode (HTMLEngineSaveState *state,
 	if (length == 0)
 		return TRUE;
 
-	encoded_buffer = encode_entities ((const guchar *) buffer, length, &encoded_length);
+	encoded_buffer = html_encode_entities ((const guchar *) buffer, length, &encoded_length);
+	success = state->receiver (state->engine, encoded_buffer, encoded_length, state->user_data);
 
-	return state->receiver (state->engine, encoded_buffer, encoded_length, state->user_data);
+	g_free (encoded_buffer);
+	return success;
 }
 
 gboolean
@@ -159,7 +161,20 @@ html_engine_save_encode_string (HTMLEngineSaveState *state,
 	return html_engine_save_encode (state, s, len);
 }
 
+gboolean
+html_engine_save_output_stringv (HTMLEngineSaveState *state,
+				 const char *format,
+				 va_list ap)
+{
+	char *string;
+	gboolean retval;
 
+	string = g_strdup_vprintf (format, ap);
+	retval = state->receiver (state->engine, string, strlen (string), state->user_data);
+	g_free (string);
+
+	return retval;
+}
 
 gboolean
 html_engine_save_output_string (HTMLEngineSaveState *state,
@@ -167,19 +182,15 @@ html_engine_save_output_string (HTMLEngineSaveState *state,
 				...)
 {
   va_list args;
-  gchar *string;
   gboolean retval;
   
   g_return_val_if_fail (format != NULL, FALSE);
   g_return_val_if_fail (state != NULL, FALSE);
   
   va_start (args, format);
-  string = g_strdup_vprintf (format, args);
+  retval = html_engine_save_output_stringv (state, format, args);
   va_end (args);
   
-  retval = state->receiver (state->engine, string, strlen (string), state->user_data);
-
-  g_free (string);
   return retval;
 }
 
@@ -215,7 +226,7 @@ get_body (HTMLEngine *e)
 		: g_strdup ("");
 	g_free (url);
 
-	body = g_strconcat ("<BODY", text, link, bg, bg_image, ">", NULL);
+	body = g_strconcat ("<BODY", text, link, bg, bg_image, ">\n", NULL);
 
 	g_free (text);
 	g_free (link);
@@ -231,6 +242,7 @@ write_header (HTMLEngineSaveState *state)
 	gboolean retval = TRUE;
 	gchar *body;
 
+	html_engine_clear_all_class_data (state->engine);
 	/* Preface.  */
 	if (! html_engine_save_output_string
 	            (state,
@@ -275,6 +287,8 @@ write_end (HTMLEngineSaveState *state)
 	if (! html_engine_save_output_string (state, "</BODY>\n</HTML>\n"))
 		return FALSE;
 
+	html_engine_clear_all_class_data (state->engine);
+
 	return TRUE;
 }
 
@@ -425,6 +439,7 @@ html_engine_save_get_paragraph_style (GtkHTMLParagraphStyle style)
 	case GTK_HTML_PARAGRAPH_STYLE_ITEMDOTTED:
 	case GTK_HTML_PARAGRAPH_STYLE_ITEMROMAN:
 	case GTK_HTML_PARAGRAPH_STYLE_ITEMDIGIT:
+	case GTK_HTML_PARAGRAPH_STYLE_ITEMALPHA:
 		return "li";
 	}
 
